]> git.tuebingen.mpg.de Git - paraslash.git/commitdiff
Teach para_fade to accept a mixer channel to fade.
authorAndre Noll <maan@systemlinux.org>
Sat, 25 Jul 2009 13:51:48 +0000 (15:51 +0200)
committerAndre Noll <maan@systemlinux.org>
Sat, 25 Jul 2009 13:51:48 +0000 (15:51 +0200)
This allows to fade not only the volume channel but all other channels that are
supported by oss.

NEWS
fade.c
ggo/fade.ggo

diff --git a/NEWS b/NEWS
index 8f225d82534b5085daab1f014a9e4651dbde63b5..6e403d92cbe528ab677b25caf85cf058fcdf4b9f 100644 (file)
--- a/NEWS
+++ b/NEWS
@@ -14,6 +14,7 @@ NEWS
        - new ls option: -d (print dates as seconds after the epoch)
        - update to gengetopt 2.22.2
        - support for RSA keys of size > 512 bits
+       - new option "mixer_channel" for para_fade
 
 -----------------------------------------
 0.3.4 (2009-05-07) "elliptic inheritance"
diff --git a/fade.c b/fade.c
index 8fb02d402f5881542409b9e22583fc4aa191b146..1e37efd5def104763d6137d32642858b9cd96d93 100644 (file)
--- a/fade.c
+++ b/fade.c
@@ -51,22 +51,22 @@ static int open_mixer(void)
 }
 
 /*
* get volume via mixer_fd
  */
-static int do_get_vol(int mixer_fd)
+static int get_mixer_channel(int mixer_fd)
 {
        int volume;
 
-       if (ioctl(mixer_fd, MIXER_READ(SOUND_MIXER_VOLUME), &volume) < 0)
+       if (ioctl(mixer_fd, MIXER_READ(conf.mixer_channel_arg), &volume) < 0)
                return -ERRNO_TO_PARA_ERROR(errno);
        /* take the mean value of left and right volume */
        return (volume % 256 + (volume >> 8)) / 2;
 }
 
 /*
- * open mixer, get volume and close mixer
+ * Open mixer, get volume and close mixer.
  */
-static int get_vol(void)
+static int open_and_get_mixer_channel(void)
 {
        int mixer_fd;
        int volume;
@@ -74,38 +74,61 @@ static int get_vol(void)
        mixer_fd = open_mixer();
        if (mixer_fd < 0)
                return mixer_fd;
-       volume = do_get_vol(mixer_fd);
+       volume = get_mixer_channel(mixer_fd);
        close(mixer_fd);
        return volume;
 }
 
 /*
* set volume via mixer_fd
  */
-static int do_set_vol(int mixer_fd, int volume)
+static int set_mixer_channel(int mixer_fd, int volume)
 {
        int tmp = (volume << 8) + volume;
 
-       if (ioctl(mixer_fd, MIXER_WRITE(SOUND_MIXER_VOLUME), &tmp) < 0)
+       if (ioctl(mixer_fd, MIXER_WRITE(conf.mixer_channel_arg), &tmp) < 0)
                return -ERRNO_TO_PARA_ERROR(errno);
        return 1;
 }
 
 /*
- * open mixer, set volume and close mixer
+ * Open mixer, set volume and close mixer.
  */
-static int set_vol(int volume)
+static int open_and_set_mixer_channel(int volume)
 {
        int mixer_fd, ret = open_mixer();
 
        if (ret < 0)
                return ret;
        mixer_fd = ret;
-       ret = do_set_vol(mixer_fd, volume);
+       ret = set_mixer_channel(mixer_fd, volume);
        close(mixer_fd);
        return ret;
 }
 
+static void fixup_mixer_channel_arg(void)
+{
+       int val;
+
+       switch (conf.mixer_channel_arg) {
+               case mixer_channel_arg_volume: val = SOUND_MIXER_VOLUME; break;
+               case mixer_channel_arg_bass: val = SOUND_MIXER_BASS; break;
+               case mixer_channel_arg_treble: val = SOUND_MIXER_TREBLE; break;
+               case mixer_channel_arg_synth: val = SOUND_MIXER_SYNTH; break;
+               case mixer_channel_arg_pcm: val = SOUND_MIXER_PCM; break;
+               case mixer_channel_arg_speaker: val = SOUND_MIXER_SPEAKER; break;
+               case mixer_channel_arg_line: val = SOUND_MIXER_LINE; break;
+               case mixer_channel_arg_mic: val = SOUND_MIXER_MIC; break;
+               case mixer_channel_arg_cd: val = SOUND_MIXER_CD; break;
+               case mixer_channel_arg_imix: val = SOUND_MIXER_IMIX; break;
+               case mixer_channel_arg_altpcm: val = SOUND_MIXER_ALTPCM; break;
+               case mixer_channel_arg_reclev: val = SOUND_MIXER_RECLEV; break;
+               case mixer_channel_arg_igain: val = SOUND_MIXER_IGAIN; break;
+               case mixer_channel_arg_ogain: val = SOUND_MIXER_OGAIN; break;
+       }
+       conf.mixer_channel_arg = val;
+}
+
 /*
  * Open mixer, get volume, fade to new_vol in secs seconds and
  * close mixer
@@ -125,7 +148,7 @@ static int fade(int new_vol, int fade_time)
        if (ret < 0)
                return ret;
        mixer_fd = ret;
-       ret = do_get_vol(mixer_fd);
+       ret = get_mixer_channel(mixer_fd);
        if (ret < 0)
                goto out;
        vol = ret;
@@ -143,7 +166,7 @@ static int fade(int new_vol, int fade_time)
                ts.tv_sec = tmp / 1000; /* really nec ?*/
                //printf("ts.tv_sec: %i\n", ts.tv_nsec);
                vol += incr;
-               ret = do_set_vol(mixer_fd, vol);
+               ret = set_mixer_channel(mixer_fd, vol);
                if (ret < 0)
                        goto out;
                //printf("vol = %i\n", vol);
@@ -226,7 +249,7 @@ static int sweet_dreams(void)
        sleep(1);
        if (sf) {
                PARA_INFO_LOG("initial volume: %d\n", iv);
-               ret = set_vol(iv);
+               ret = open_and_set_mixer_channel(iv);
                if (ret < 0)
                        return ret;
                change_afs_mode_and_play(fa_mode);
@@ -234,7 +257,7 @@ static int sweet_dreams(void)
                if (ret < 0)
                        return ret;
        } else {
-               ret = set_vol(sf);
+               ret = open_and_set_mixer_channel(sf);
                if (ret < 0)
                        return ret;
        }
@@ -268,8 +291,8 @@ static int snooze(void)
        if (conf.snooze_time_arg <= 0)
                return 1;
        sleep_time = conf.snooze_time_arg;
-       if (get_vol() < conf.snooze_out_vol_arg)
-               ret = set_vol(conf.snooze_out_vol_arg);
+       if (open_and_get_mixer_channel() < conf.snooze_out_vol_arg)
+               ret = open_and_set_mixer_channel(conf.snooze_out_vol_arg);
        else
                ret = fade(conf.snooze_out_vol_arg, conf.snooze_out_fade_arg);
        if (ret < 0)
@@ -318,6 +341,7 @@ int main(int argc, char *argv[])
                fade_cmdline_parser_config_file(conf.config_file_arg,
                        &conf, &params);
        }
+       fixup_mixer_channel_arg();
        if (!strcmp(conf.mode_arg, "sleep")) {
                ret = sweet_dreams();
                goto out;
index 388642e5c60a8f6936cd383445ea34bac67967af..01eb6aa98efdb801bab2718917eeb3186ad203c8 100644 (file)
@@ -33,6 +33,18 @@ option "mixer_device" m
        default="/dev/mixer"
        optional
 
+option "mixer_channel" C
+#~~~~~~~~~~~~~~~~~~~~~~~
+"select the analog mixer channel"
+       enum typestr="channel"
+       values="volume", "bass", "treble", "synth", "pcm", "speaker", "line",
+               "mic", "cd", "imix", "altpcm", "reclev", "igain", "ogain"
+       default="volume"
+       optional
+       details="
+       Not all listed channels might be supported on any particular hardware.
+"
+
 section "sleep options (only relevant in sleep mode)"
 #####################################################
@@ -51,22 +51,22 @@ static int open_mixer(void)
 
 /*
- * get volume via mixer_fd
+ * Get channel volume via mixer_fd.
  */
@@ -74,38 +74,61 @@ static int get_vol(void)
 
 /*
- * set volume via mixer_fd
+ * Set channel volume via mixer_fd.
  */